# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient.auth.identity import v3
from keystoneclient import session
from keystoneclient.v3 import client as ks_client
from oslo_config import cfg

CONF = cfg.CONF


def _get_trustor_session(token, project_id):
    """Return a token-based session for authenticated user."""

    settings = _get_keystone_settings()

    client_auth = v3.Token(
        token=token,
        project_id=project_id,
        auth_url=settings['auth_url'],
    )
    return session.Session(auth=client_auth)


def _get_service_user_session():
    """
    Return a session for service user by the settings in authtoken_middleware.
    """
    settings = _get_keystone_settings()

    service_user_auth = v3.Password(
        project_name=settings['project_name'],
        username=settings['username'],
        password=settings['password'],
        auth_url=settings['auth_url'],
        user_domain_id=settings['user_domain_id'],
        user_domain_name=settings['user_domain_name'],
        project_domain_id=settings['project_domain_id'],
        project_domain_name=settings['project_domain_name'],
    )
    return session.Session(auth=service_user_auth)


def _get_trustee_session(trust_id):
    """
    Return a session for service user as a trustee.
    """
    settings = _get_keystone_settings()

    trustee_auth = v3.Password(
        trust_id=trust_id,
        username=settings['username'],
        password=settings['password'],
        auth_url=settings['auth_url'],
        user_domain_id=settings['user_domain_id'],
        user_domain_name=settings['user_domain_name'],
    )
    return session.Session(auth=trustee_auth)


def create_client_for_trust(token, project_id, roles):
    """Create a keystone client with a trust for the service user.

    :param token: valid user token
    :param project_id: ID of the project the user has a role on
    :param roles: list of roles of user in the project
    """
    settings = _get_keystone_settings()
    ssl = settings['ssl']

    # get trustor id
    trustor_sess = _get_trustor_session(token, project_id)
    trustor_client = ks_client.Client(session=trustor_sess, **ssl)
    trustor_id = trustor_client.session.get_user_id()

    # get service user id
    service_user_sess = _get_service_user_session()
    service_user_client = ks_client.Client(session=service_user_sess, **ssl)
    trustee_id = service_user_client.session.get_user_id()

    # create a trust for service user
    trust = trustor_client.trusts.create(trustor_user=trustor_id,
                                         trustee_user=trustee_id,
                                         impersonation=True,
                                         role_names=roles,
                                         project=project_id)

    # create an instance of keystone client with trust
    trustee_sess = _get_trustee_session(trust_id=trust.id)
    return ks_client.Client(session=trustee_sess, **ssl)


def get_trust_id(keystone_client):
    """Get trust id for instance of keystone client.

    :param keystone_client: instance of keystone client with trust-scoped auth
    """
    return keystone_client.session.auth.trust_id


def delete_trust(keystone_client):
    """Delete previously created trust.

    :param keystone_client: instance of keystone client with trust-scoped auth
    """
    keystone_client.trusts.delete(get_trust_id(keystone_client))


def renew_token(keystone_client):
    """Get new valid token.

    :param keystone_client: instance of keystone client with trust-scoped auth
    """
    return keystone_client.session.get_token()


def _get_keystone_settings():
    return {
        'auth_url': CONF.keystone_authtoken.auth_uri + '/v3',
        'username': CONF.keystone_authtoken.admin_user,
        'password': CONF.keystone_authtoken.admin_password,
        'project_name': CONF.keystone_authtoken.admin_tenant_name,
        'user_domain_id': "default",
        'user_domain_name': "Default",
        'project_domain_id': "default",
        'project_domain_name': "Default",
        'ssl': {
            'cacert': CONF.keystone_authtoken.cafile,
            'insecure': CONF.keystone_authtoken.insecure,
            'cert': CONF.keystone_authtoken.certfile,
            'key': CONF.keystone_authtoken.keyfile,
        }
    }
